<?php

namespace Drupal\skilling\Migrate;

use Drupal\Core\Field\EntityReferenceFieldItemListInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\file\Plugin\Field\FieldType\FileFieldItemList;
use Drupal\file\Plugin\Field\FieldType\FileItem;
use Drupal\views\Plugin\views\row\EntityReference;

/**
 * Class MigrateExport.
 */
class MigrateExport {

  use MigrateTrait;
  use StringTranslationTrait;

  /**
   * Constructs a new MigrateExport object.
   */
  public function __construct(
    MessengerInterface $messenger,
    EntityTypeManagerInterface $entityTypeManager,
    FileSystemInterface $fileSystem
  ) {
    // Call trait method to remember the services.
    $this->storeServices($messenger, $entityTypeManager, $fileSystem);
    $this->createPath($this->migratePath);
    $this->createPath($this->migratePath . 'files');
  }

  public function exportLessons() {
    $lessonNids = $this->entityTypeManager->getStorage('node')
      ->getQuery()
      ->condition('type', 'lesson')
      ->execute();
    $numberLessonsFound = count($lessonNids);
    if ($numberLessonsFound > 0) {
      $lessons = $this->entityTypeManager->getStorage('node')
        ->loadMultiple($lessonNids);
      $allLessonDeets = [];
      /** @var \Drupal\node\NodeInterface $lesson */
      foreach ($lessons as $lesson) {
        $lessonDeets = [
          'sourceId' => $lesson->id(),
          'title' => $lesson->getTitle(),
          'sourceOwnerId' => $lesson->getOwnerId(),
          'published' => $lesson->isPublished(),
          'bodySummary' => $lesson->body->summary,
          'bodyFull' => $lesson->body->value,
          'fieldShowToc' => $lesson->field_show_toc->value,
          'fieldOrderInBook' => $lesson->field_order_in_book->value,
          'fieldNotes' => $lesson->field_notes->value,
        ];
         // Get the account name of the owner.
        /** @var \Drupal\user\UserInterface $user */
        $user = $this->entityTypeManager->getStorage('user')->
          load($lesson->getOwnerId());
        if ($user) {
          $userName = $user->getAccountName();
        }
        else {
          $userName = $this->t('(Unknown)');
        }
        $lessonDeets['sourceOwnerName'] = $userName;
//        $tags = $lesson->field_tags->getValue();
//        foreach ($tags as $tag) {
//          $lessonDeets['fieldTags'][] = ['termId' => $tag['target_id']];
//        }
        $attachments = $lesson->field_attachments;
        $attachedFileDeets = $this->extractFileFieldItemList($attachments);
        $hiddenAttachments = $lesson->field_hidden_attachments;
        $hiddenAttachedFileDeets = $this->extractFileFieldItemList($hiddenAttachments);
        $lessonDeets['attachments'] = $attachedFileDeets;
        $lessonDeets['hiddenAttachments'] = $hiddenAttachedFileDeets;
        // Export terms.
        $lessonDeets['fieldTags'] = $this->exportTags($lesson->field_tags);
        $allLessonDeets[] = $lessonDeets;
      }
      $json = json_encode($allLessonDeets);
      $jsonPath = $this->migratePath . 'lessons.json';
      $stringToFileResult = $this->stringToFile($json, $jsonPath);
      if (!$stringToFileResult) {
        $this->messenger->addError('Could not output ' . $jsonPath);
      }
    }
    return $numberLessonsFound;
  }


  protected function exportTags(EntityReferenceFieldItemListInterface $tags) {
    $r = 6;
    $result = [];
    /** @var \Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem $tag */
    foreach ($tags as $tag) {
      $termId = $tag->get('target_id')->getValue();
      /** @var \Drupal\taxonomy\Entity\Term $term */
      $term = $this->entityTypeManager->getStorage('taxonomy_term')->load($termId);
      $termName = $term->getName();
      $vocabId = $term->bundle();
      $result[] = [
        'term' => $termName,
        'taxonomy' => $vocabId,
      ];
      $r  =6;
    }
    return $result;
  }

  protected function stringToFile(string $string, string $filePath) {
    $result = file_put_contents($filePath, $string);
    if ($result) {
      $this->fileSystem->chmod($filePath);
    }
    return $result;
  }

  protected function extractFileFieldItemList(FileFieldItemList $fileFieldItemList) {
    $fileItemDeets = [];
    foreach ($fileFieldItemList as $fileFieldItem) {
      // Get the deets of the file field item.
      $fileDeets = $this->extractFileEntityReferenceItemDeets($fileFieldItem);
      // Copy the file itself.
      $migratedPath = $this->copyFileForMigration($fileDeets['sourceUri']);
      // Remember where it was copied to.
      $fileDeets['migratedPath'] = $migratedPath;
      $fileItemDeets[] = $fileDeets;
    }
    return $fileItemDeets;
  }

  protected function extractFileEntityReferenceItemDeets(FileItem $fileItem) {
    $fileId = $fileItem->getValue()['target_id'];
    /** @var \Drupal\file\Entity\File $file */
    $file = $this->entityTypeManager->getStorage('file')
      ->load($fileId);
    $fileDeets = [
      'description' => $fileItem->getValue()['description'],
      'sourceId' => $fileId,
      'sourceUri' => $file->getFileUri(),
      'sourceFileName' => $file->getFilename(),
      'mimeType' => $file->getMimeType(),
      'ownerId' => $file->getOwnerId(),
    ];
    return $fileDeets;
  }

  protected function copyFileForMigration(string $sourceUri) {
    $source = $this->fileSystem->realpath($sourceUri) ?: $sourceUri;
    $destination = $this->migratePath . 'files/' . $this->migrateFilePrefix . $this->fileCounter;
    $this->fileCounter ++;
    // Perform the copy operation.
    if (!@copy($source, $destination)) {
      $this->messenger->addError('Could not copy ' . $sourceUri);
      return FALSE;
    }
    // Set the permissions on the new file.
    $this->fileSystem->chmod($destination);
    return $destination;
  }

}
