<?php

namespace Drupal\skilling\Migrate;

use Drupal\Core\Entity\EntityTypeBundleInfo;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\file\Plugin\Field\FieldType\FileFieldItemList;
use Drupal\file\Plugin\Field\FieldType\FileItem;
use Drupal\skilling\SkillingConstants;

/**
 * Class MigrateExport.
 */
class MigrateImport {

  use MigrateTrait;
  use StringTranslationTrait;

  protected $entityTypeBundleInfoService;

  /**
   * Constructs a new MigrateExport object.
   */
  public function __construct(
    MessengerInterface $messenger,
    EntityTypeManagerInterface $entityTypeManager,
    FileSystemInterface $fileSystem,
    EntityTypeBundleInfo $entityTypeBundleInfoService
  ) {
    // Call trait method to remember the services.
    $this->storeServices($messenger, $entityTypeManager, $fileSystem);
    $this->createPath($this->migratePath);
    $this->createPath($this->migratePath . 'files');
    $this->entityTypeBundleInfoService = $entityTypeBundleInfoService;
  }

  public function import(bool $justTesting) {
    $this->importLessons($justTesting);
  }

  /**
   * @return int
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\skilling\Migrate\SkillingMigrationException
   */
  protected function importLessons(bool $justTesting) {
    // Import lesson JSON.
    $jsonPath = $this->migratePath . 'lessons.json';
    $jsonString = file_get_contents($jsonPath);
    $jsonLessons = json_decode($jsonString);
    foreach ($jsonLessons as $jsonLesson) {
      $title = $jsonLesson->title;
      if (!$title) {
        $this->messenger->addWarning(
          $this->t('Skipping lesson with no title.')
        );
        continue;
      }
      $bodyFull = $jsonLesson->bodyFull;
      $bodySummary = $jsonLesson->bodySummary;
      $sourceOwnerName = $jsonLesson->sourceOwnerName;
      $destinationOwnerId = $this->getUserIdWithAccountName($sourceOwnerName);
      if (!$destinationOwnerId) {
        $warning = $this->t(
          "Lesson with title: '@t'.<br><br>Owner user '@u' not found. Assigning to user 1.",
          ['@t' => $title, '@u' => $sourceOwnerName]
        );
        $this->messenger->addWarning($warning);
        $destinationOwnerId = 1;
      }
      $published = $jsonLesson->published;
      $fieldShowToc = $jsonLesson->fieldShowToc;
      $fieldOrderInBook = $jsonLesson->fieldOrderInBook;
      $fieldNotes = $jsonLesson->fieldNotes;
      $termIds = $this->updateTaxomonies($jsonLesson->fieldTags, $justTesting);
      // Create a lesson node.
      /** @var \Drupal\node\NodeInterface $lesson */
      $lesson = $this->entityTypeManager
        ->getStorage('node')
        ->create(['type' => SkillingConstants::LESSON_CONTENT_TYPE]);
      // Set base properties.
      $lesson->setTitle($title);
      $lesson->setOwnerId($destinationOwnerId);
      $lesson->setPublished($published);
      // Set fields.
      $lesson->set(
        SkillingConstants::FIELD_BODY,
        [
          'value' => $bodyFull,
          'summary' => $bodySummary,
          'format' => SkillingConstants::SKILLING_TEXT_FORMAT,
        ]
      );
      $lesson->set(SkillingConstants::FIELD_SHOW_TOC, $fieldShowToc);
      $lesson->set(SkillingConstants::FIELD_ORDER_IN_BOOK, $fieldOrderInBook);
      if ($fieldNotes) {
        $lesson->set(
          SkillingConstants::FIELD_NOTES,
          [
            'value' => $fieldNotes,
            'format' => SkillingConstants::STRIPPED_TEXT_FORMAT,
          ]
        );

      }
      $lesson->save();
    }



    return count($jsonLessons);



    $lessonNids = $this->entityTypeManager->getStorage('node')
      ->getQuery()
      ->condition('type', 'lesson')
      ->execute();
    $numberLessonsFound = count($lessonNids);
    if ($numberLessonsFound > 0) {
      $lessons = $this->entityTypeManager->getStorage('node')
        ->loadMultiple($lessonNids);
      $allLessonDeets = [];
      /** @var \Drupal\node\NodeInterface $lesson */
      foreach ($lessons as $lesson) {
        $lessonDeets = [
          'sourceId' => $lesson->id(),
          'title' => $lesson->getTitle(),
          'sourceOwnerId' => $lesson->getOwnerId(),
          'published' => $lesson->isPublished(),
          'bodySummary' => $lesson->body->summary,
          'bodyFull' => $lesson->body->value,
          'fieldShowToc' => $lesson->field_show_toc->value,
          'fieldOrderInBook' => $lesson->field_order_in_book->value,
          'fieldNotes' => $lesson->field_notes->value,
        ];
        $tags = $lesson->field_tags->getValue();
        foreach ($tags as $tag) {
          $lessonDeets['fieldTags'][] = ['termId' => $tag['target_id']];
        }
        $attachments = $lesson->field_attachments;
        $attachedFileDeets = $this->extractFileFieldItemList($attachments);
        $hiddenAttachments = $lesson->field_hidden_attachments;
        $hiddenAttachedFileDeets = $this->extractFileFieldItemList($hiddenAttachments);
        $lessonDeets['attachments'] = $attachedFileDeets;
        $lessonDeets['hiddenAttachments'] = $hiddenAttachedFileDeets;
        $allLessonDeets[] = $lessonDeets;
      }
      $json = json_encode($allLessonDeets);
      $jsonPath = $this->migratePath . 'lessons.json';
      $stringToFileResult = $this->stringToFile($json, $jsonPath);
      if (!$stringToFileResult) {
        $this->messenger->addError('Could not output ' . $jsonPath);
      }
    }
    return $numberLessonsFound;
  }


  /**
   * Update the taxonomies for an entity.
   *
   * Takes an array of items, each with a term, and a taxomomy id (its
   * machine name). Makes the terms it does not find. Returns the
   * term ids of the corresponding terms.
   *
   * @param array $importedTerms
   *   The terms.
   * @param bool $justTesting
   *   If true, no DB updates are made.
   *
   * @return array
   *   Array of term ids. MT if just testing.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function updateTaxomonies(array $importedTerms, bool $justTesting) {
    $destinationSiteTermIds = [];
    foreach ($importedTerms as $importedTerm) {
      $termText = $importedTerm->term;
      $taxonomy = $importedTerm->taxonomy;
      // Is that term in that vocab?
      $term = $this->entityTypeManager->getStorage('taxonomy_term')
        ->loadByProperties([
          'name' => $termText,
          'vid' => $taxonomy,
        ]);
      if ($term) {
        // There is a matching term on this (the destination) site.
        $destinationSiteTermIds[] = reset($term)->id();
      }
      else {
        // Create the term.
        // Does the taxonomy exist?
        $taxonomyBundles = $this->entityTypeBundleInfoService->getBundleInfo('taxonomy_term');
        if (!in_array($taxonomy, array_keys($taxonomyBundles))) {
          // Taxonomy does not exist.
          $message = $this->t(
            "Taxonomy with machine name @n not found. Create it, and try the import again.",
            ['@n' => $taxonomy]
          );
          $this->messenger->addError($message);
        }
        else {
          // Taxonomy does exist. Add the term to it.
          if ($justTesting) {
            $message = $this->t(
              'Migration would have created the term "@term" in the taxonomy "@tax", if this was for real.',
              ['@term' => $termText, '@tax' => $taxonomy]
            );
          }
          else {
            /** @var \Drupal\taxonomy\Entity\Term $term */
            $term = $this->entityTypeManager->getStorage('taxonomy_term')
              ->create([
                'name' => $termText,
                'vid' => $taxonomy,
              ]);
            $term->save();
            $message = $this->t(
              'Created term "@term" in the taxonomy "@tax".',
              ['@term' => $termText, '@tax' => $taxonomy]
            );
            // Add new term id to list of terms to use.
            $destinationSiteTermIds[] = $term->id();
          } // Not just testing.
          $this->messenger->addStatus($message);
        } // End the taxonomy exists.
      } // End no matching term.
    } // End foreach imported term.
    return $destinationSiteTermIds;
  }

  protected function getUserIdWithAccountName(string $accountName) {
    $uids = $this->entityTypeManager->getStorage('user')
      ->getQuery()
      ->condition('name', $accountName)
      ->execute();
    if (count($uids) === 0) {
      return FALSE;
    }
    $uid = reset($uids);
    return $uid;
  }

  protected function isUserIdExists(int $id) {
    if (!is_numeric($id)) {
      return FALSE;
    }
    if ($id < 1) {
      return FALSE;
    }
    $count = $this->entityTypeManager->getStorage('user')
      ->getQuery()
      ->condition('uid', $id)
      ->count()
      ->execute();
    return $count > 0;
  }

  protected function stringToFile(string $string, string $filePath) {
    $result = file_put_contents($filePath, $string);
    if ($result) {
      $this->fileSystem->chmod($filePath);
    }
    return $result;
  }

  protected function extractFileFieldItemList(FileFieldItemList $fileFieldItemList) {
    $fileItemDeets = [];
    foreach ($fileFieldItemList as $fileFieldItem) {
      // Get the deets of the file field item.
      $fileDeets = $this->extractFileEntityReferenceItemDeets($fileFieldItem);
      // Copy the file itself.
      $migratedPath = $this->copyFileForMigration($fileDeets['sourceUri']);
      // Remember where it was copied to.
      $fileDeets['migratedPath'] = $migratedPath;
      $fileItemDeets[] = $fileDeets;
    }
    return $fileItemDeets;
  }

  protected function extractFileEntityReferenceItemDeets(FileItem $fileItem) {
    $fileId = $fileItem->getValue()['target_id'];
    /** @var \Drupal\file\Entity\File $file */
    $file = $this->entityTypeManager->getStorage('file')
      ->load($fileId);
    $fileDeets = [
      'description' => $fileItem->getValue()['description'],
      'sourceId' => $fileId,
      'sourceUri' => $file->getFileUri(),
      'mimeType' => $file->getMimeType(),
      'ownerId' => $file->getOwnerId(),
    ];
    return $fileDeets;
  }

  protected function copyFileForMigration(string $sourceUri) {
    $source = $this->fileSystem->realpath($sourceUri) ?: $sourceUri;
    $destination = $this->migratePath . 'files/' . $this->migrateFilePrefix . $this->fileCounter;
    $this->fileCounter++;
    // Perform the copy operation.
    if (!@copy($source, $destination)) {
      $this->messenger->addError('Could not copy ' . $sourceUri);
      return FALSE;
    }
    // Set the permissions on the new file.
    $this->fileSystem->chmod($destination);
    return $destination;
  }

}
